/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "new.h"

// This is the default constructor for the new project dialog. It is meant
// to initialize all of the objects and set their properties. It then creates
// the dialog and displays it to the screen
NewProjectDialog::NewProjectDialog (TemplateList *list_of_templates)
{
  tlist = list_of_templates;
  initialize_objects();
  populate_combobox();

  string temp = string(g_get_home_dir()) + "/openldev/";
  gtk_entry_set_text(GTK_ENTRY(location), temp.c_str());

  g_signal_connect(G_OBJECT(apply), "clicked", G_CALLBACK(on_apply_pressed), (gpointer)this);
  g_signal_connect(G_OBJECT(cancel), "clicked", G_CALLBACK(on_cancel_pressed), (gpointer)this);
  g_signal_connect(G_OBJECT(browse), "clicked", G_CALLBACK(on_browse_pressed), (gpointer)this);
}

// The dialog and its widgets are initialized in this function.
void NewProjectDialog::initialize_objects()
{
  string gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/newproject.glade";
  GladeXML *xml = glade_xml_new(gladefile.c_str(), NULL, NULL);

  dialog = glade_xml_get_widget(xml, "dialog");
  combobox = glade_xml_get_widget(xml, "comboboxentry");
  name = glade_xml_get_widget(xml, "name");
  version = glade_xml_get_widget(xml, "version");
  homepage = glade_xml_get_widget(xml, "homepage");
  email = glade_xml_get_widget(xml, "email");
  author = glade_xml_get_widget(xml, "author");
  location = glade_xml_get_widget(xml, "location");
  cancel = glade_xml_get_widget(xml, "button_cancel");
  apply = glade_xml_get_widget(xml, "button_ok");
  browse = glade_xml_get_widget(xml, "button_browse");
}

GtkWidget* NewProjectDialog::get_dialog()
{ return dialog;
}

// When cancel is pressed, this function is called to emit a cancel signal
// and hide the dialog
void on_cancel_pressed (GtkWidget *widget, 
                        gpointer data)
{
  NewProjectDialog *ptr = (NewProjectDialog*) data;
  gtk_dialog_response(GTK_DIALOG(ptr->dialog), GTK_RESPONSE_CANCEL);
  gtk_widget_hide(ptr->dialog);
}

// When the browse button is pressed, the user is presented with a 
// FileChooserDialog where they can choose where to create the project.
void on_browse_pressed (GtkWidget *widget, 
                        gpointer data)
{
  NewProjectDialog *ptr = (NewProjectDialog*) data;
  
  GtkWidget *filedialog;
  filedialog = gtk_file_chooser_dialog_new("Choose Directory", NULL,
                GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, 
                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
                GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
  
  int result = gtk_dialog_run(GTK_DIALOG(filedialog));
  switch (result)
  {
    // If OK is pressed
    case (GTK_RESPONSE_ACCEPT):
    {
      string fn = string(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filedialog))) + "/";
      gtk_entry_set_text(GTK_ENTRY(ptr->location), fn.c_str());
      break;
    }
    default:
      break;
  }
  
  gtk_widget_hide(filedialog);
}

// Check the dialog for the following errors: If the project name is empty or
// contains non-alphanumeric characters, if the version is empty, if the project
// locatino is empty
string NewProjectDialog::check_for_errors()
{
  string ntxt = gtk_entry_get_text(GTK_ENTRY(name)),
         vtxt = gtk_entry_get_text(GTK_ENTRY(version)),
         ltxt = gtk_entry_get_text(GTK_ENTRY(location));
  
  // Check for error when creating a project or file
  if (ntxt.find(" ",0) != string::npos ||
      ntxt.find("_",0) != string::npos ||
      ntxt.empty())
    return "The project name must\nbe alphanumeric!";
  else if (vtxt.empty())
    return "You must specify a\nversion number!";
  else if (ltxt.empty())
    return "You must specify a\nproject location!";
  else
    return "NONE";
}

// Create the directory the project is to be created in
void NewProjectDialog::create_location()
{
  string str = gtk_entry_get_text(GTK_ENTRY(location));
  mkdir(str.c_str(), 0777);
  str += gtk_entry_get_text(GTK_ENTRY(name)) + string("/");
  mkdir(str.c_str(), 0777);
  str += "src/";
  mkdir(str.c_str(), 0777);
}

// Create a new project file. This function returns text that is to be added
// to the project file and returns by reference the location the project file
// is to be created.
string NewProjectDialog::create_odp_file_text(string &dir)
{
  string choice_text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(combobox)),
         odp_file_text, sources;
  
  if (choice_text.find("C++", 0) != string::npos)
    sources = "main.cc/";
  else
    sources = "main.c/";
    
  
  // Create the text for the file
  odp_file_text = "<name>" + string(gtk_entry_get_text(GTK_ENTRY(name))) 
    + "*\n<author>" + gtk_entry_get_text(GTK_ENTRY(author))
    + "*\n<location>" + gtk_entry_get_text(GTK_ENTRY(location)) 
    + gtk_entry_get_text(GTK_ENTRY(name)) + "/*\n<email>" 
    + gtk_entry_get_text(GTK_ENTRY(email)) + "*\n<url>" 
    + gtk_entry_get_text(GTK_ENTRY(homepage)) + "*\n<version>" 
    + gtk_entry_get_text(GTK_ENTRY(version));
  
  // Add the files that need to be added for the template
  for (unsigned int i=0; i < tlist->title.size(); i++)
  {
    if (tlist->title.at(i) == choice_text)
    {
      odp_file_text += string("*\n<template>") + tlist->file.at(i) 
                    + "*\n<lastfile>" + sources.substr(0, sources.find("/", 0))
                    + "*\n<sources>" + sources
                    + "*\n<headers>*\n<otherfiles>*\n<execname>";
    }
  }
    
  dir = gtk_entry_get_text(GTK_ENTRY(location)) + string(gtk_entry_get_text(GTK_ENTRY(name))) 
      + "/" + gtk_entry_get_text(GTK_ENTRY(name)) + ".openldev";
  
  return odp_file_text;
}

void NewProjectDialog::clean_white_space_and_replace_content (string &text, 
                                                              bool isMakefile)
{
  string firststring[] = { "AUTHOR", "HOMEPAGE", "EMAIL", "TITLE", "VERSION", "[[", "]]", "AMP" };
  string replacement[] = { gtk_entry_get_text(GTK_ENTRY(author)), 
         gtk_entry_get_text(GTK_ENTRY(homepage)), gtk_entry_get_text(GTK_ENTRY(email)), 
         gtk_entry_get_text(GTK_ENTRY(name)), gtk_entry_get_text(GTK_ENTRY(version)), "<", ">", "&" };
  
  for (unsigned int i=0; i<8; i++)
    while ((text.find(firststring[i], 0) != string::npos))
    {
      if (!(i == 0 && isMakefile))
      {
        int l = text.find(firststring[i], 0);
        text.erase(l, firststring[i].length());
        text.insert(l, replacement[i]);
      }
      else
        break;
    }
}

void NewProjectDialog::create_project_from_template (string filename)
{
  string file = string(INSTALL_PREFIX) + "/share/openldev/templates/"
                     + filename + ".template";
  xmlDocPtr doc = xmlParseFile (file.c_str());
  xmlNodePtr cur;
  
  if (doc != NULL) 
  {
    cur = xmlDocGetRootElement(doc);
    if (!xmlStrcmp(cur->name, (const xmlChar *) "project"))
    {
      cur = cur->xmlChildrenNode;
      while (cur != NULL)
      {
        if (!xmlStrcmp(cur->name, (const xmlChar *) "basefiles"))
        {
          cur = cur->xmlChildrenNode;
          while (cur != NULL)
          {
            if (!xmlStrcmp(cur->name, (const xmlChar *) "file"))
            {
              string fn = string(gtk_entry_get_text(GTK_ENTRY(location)))
                        + gtk_entry_get_text(GTK_ENTRY(name)) + "/" 
                        + (const char *)xmlGetProp(cur, (const xmlChar *) "name");
              string content = (const char *)xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
              
              if (content.find("AUTHORS", 0) != string::npos)
                clean_white_space_and_replace_content(content, true);
              else
                clean_white_space_and_replace_content(content, false);
              write_file(fn, content);
              
              if (content.find("#!/bin/sh", 0) != string::npos)
                chmod(fn.c_str(), 0777);
            }
            
            cur = cur->next;
          }

          break;
        }
        cur = cur->next;
      }
      
      cur = xmlDocGetRootElement(doc);
      cur = cur->xmlChildrenNode;
      while (cur != NULL)
      {
        if (!xmlStrcmp(cur->name, (const xmlChar *) "srcfiles"))
        {
          cur = cur->xmlChildrenNode;
          while (cur != NULL)
          {
            if (!xmlStrcmp(cur->name, (const xmlChar *) "file"))
            {
              string fn = string(gtk_entry_get_text(GTK_ENTRY(location))) 
                        + gtk_entry_get_text(GTK_ENTRY(name)) + "/src/" 
                        + (const char *)xmlGetProp(cur, (const xmlChar *) "name");
              string content = (const char *) xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
              clean_white_space_and_replace_content(content, false);
              write_file(fn, content);
              if (content.find("#!/bin/sh", 0) != string::npos)
                chmod(fn.c_str(), 0777);
            }
            
            cur = cur->next;
          }
          
          break;
        }
        cur = cur->next;
      }
    }
  }
  
  xmlFreeDoc(doc);
}

// When the apply button is pressed, it is checked that the project name only
// contains letters and numbers and that a version and location is specified.
// The project is then created from a template and the Makefiles and configure
// file are edited. The OpenLDev project file is also created.
void on_apply_pressed (GtkWidget *widget, 
                       gpointer data)
{
  NewProjectDialog *ptr = (NewProjectDialog*) data;  
  string errortext = ptr->check_for_errors();
  
  // Create new project
  if (errortext == "NONE")
  {
    string odp_file_text, odp_file_location, project_path, copy, extract,
                  rm, configure_in, makefile_am, makefile_am2, text;
    // Add a '/' to the end of the location if it does not exist
    string locationtext = gtk_entry_get_text(GTK_ENTRY(ptr->location));
    if (locationtext.at(locationtext.length()-1) != '/')
    {
      string str = locationtext;
      str += "/";
      gtk_entry_set_text(GTK_ENTRY(ptr->location), str.c_str());
    }

    // Create the project location, openldev file location and openldev file text
    ptr->create_location();
    odp_file_text = ptr->create_odp_file_text(odp_file_location);    
    
    // Create directory path
    project_path += string(gtk_entry_get_text(GTK_ENTRY(ptr->location))) 
                 + gtk_entry_get_text(GTK_ENTRY(ptr->name)) + "/";
    
    // Create copy command, configure.in path, & makefile.am path
    string choice_text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(ptr->combobox));
    
    for (unsigned int i=0; i < ptr->tlist->file.size(); i++)
      if (ptr->tlist->title.at(i) == choice_text)
      {
        ptr->create_project_from_template(ptr->tlist->file.at(i));
        i = ptr->tlist->file.size();
      }
    
    // Create the project from the template & create the project file
    write_file(odp_file_location, odp_file_text);
    ptr->file_location = odp_file_location;
    
    gtk_dialog_response(GTK_DIALOG(ptr->dialog), GTK_RESPONSE_ACCEPT);
    gtk_widget_hide(ptr->dialog);
  }
  // Throw an error
  else
  {
    error(errortext);
  }
}

// This adds the different types of projects to the combobox
void NewProjectDialog::populate_combobox()
{ 
  GtkTreeModel *model;
	GtkTreeIter iter;
  GtkCellRenderer *renderer;

	model = GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
	
	for (unsigned int i=0; i < tlist->title.size(); i++) 
  {
		gtk_list_store_append(GTK_LIST_STORE(model), &iter);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, tlist->title.at(i).c_str(), -1);
	}
  
  gtk_combo_box_set_model(GTK_COMBO_BOX(combobox), model);
	g_object_unref(model);
  
  renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combobox), renderer, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combobox), renderer, "text", 0, NULL);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), 0);
}

// This allows the main openldev class access to the filename of the project.
string NewProjectDialog::get_filename() 
{ return file_location; 
}
