/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "newfile.h"

// This is the default constructor for the new file dialog. It is meant
// to initialize all of the objects and set their properties. It then creates
// the dialog and displays it to the screen. It receives the project settings
// object so the location of project files can be accessed.
NewFileDialog::NewFileDialog (ProjectSettings *proj_settings)
{
  file_location = proj_settings->get_location() + "src/";
  
  initialize_objects();
  populate_combobox();
  
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(checkbutton), true);
  
  g_signal_connect(G_OBJECT(apply), "clicked", 
                   G_CALLBACK(on_apply), (gpointer)this);
  g_signal_connect(G_OBJECT(cancel), "clicked", 
                   G_CALLBACK(on_cancel), (gpointer)this);
}

// The dialog and its widgets are initialized in this function.
void NewFileDialog::initialize_objects()
{
  string gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/newfile.glade";
  GladeXML *xml = glade_xml_new(gladefile.c_str(), NULL, NULL);
  
  dialog = glade_xml_get_widget(xml, "dialog");
  cancel = glade_xml_get_widget(xml, "button_cancel");
  apply = glade_xml_get_widget(xml, "button_ok");
  combobox = glade_xml_get_widget(xml, "filetypes");
  name = glade_xml_get_widget(xml, "filename");
  checkbutton = glade_xml_get_widget(xml, "checkbutton");
}

GtkWidget* NewFileDialog::get_dialog()
{ return dialog;
}

// When cancel is pressed, this function is called to emit a cancel signal
// and hide the dialog
void on_cancel (GtkWidget *widget, 
                gpointer data)
{
  NewFileDialog *ptr = (NewFileDialog*) data;
  gtk_dialog_response(GTK_DIALOG(ptr->dialog), GTK_RESPONSE_CANCEL);
  gtk_widget_hide(ptr->dialog);
}

// When the apply button is pressed, it is checked that the file name only
// contains letters and numbers and that a version and location is specified.
// The file is then created in the project's source directory
void on_apply (GtkWidget *widget, 
               gpointer data)
{
  NewFileDialog *ptr = (NewFileDialog*) data;
  string choice_text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(ptr->combobox));
  string ntxt = gtk_entry_get_text(GTK_ENTRY(ptr->name));
  
  // Create new file if no errors are found
  if (!(ntxt.find(" ",0) != string::npos || ntxt.empty()))
  {
    string ext1="~", ext2="~", ext3="~";
    string projectname = gtk_entry_get_text(GTK_ENTRY(ptr->name));
    ptr->file_location += projectname;
    
    // Create possible file extensions
    if (choice_text == "C Source File (.c)") {
      ext1 = ".c";
    } 
    else if (choice_text == "C Header File (.h)")  {
      ext1 = ".h";
    } 
    else if (choice_text == "C++ Header File (.hh)")  {
      ext1 = ".hh";
    }
    else if (choice_text == "C++ Source File (.cc)")  {
      ext1 = ".cc";
      ext2 = ".cpp";
      ext3 = ".cxx";
    }
    
    // Add extension if it is not there
    if (projectname.find(ext1,0) == string::npos && 
        projectname.find(ext2,0) == string::npos && 
        projectname.find(ext3,0) == string::npos)
      if (ext1 != "~")
        ptr->file_location += ext1;
    
    // Write new file
    write_file(ptr->file_location, "");
    
    gtk_dialog_response(GTK_DIALOG(ptr->dialog), GTK_RESPONSE_ACCEPT);
    gtk_widget_hide(ptr->dialog);
  }
  // Throw an error
  else 
    error("You must specify a\nfile name!");
}

// This adds the different types of files to the combobox
void NewFileDialog::populate_combobox()
{
	GtkTreeModel *model;
	GtkTreeIter iter;
  GtkCellRenderer *renderer;
	gchar *types[] = {
		"C Source File (.c)",
    "C Header File (.h)",
    "C++ Source File (.cc)",
    "C++ Header File (.hh)",
    "Other File",
		NULL
	};

	model = GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
	
	for (int i=0; types[i]; i++) 
  {
		gtk_list_store_append(GTK_LIST_STORE(model), &iter);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, types[i], -1);
	}
  
  gtk_combo_box_set_model(GTK_COMBO_BOX(combobox), model);
	g_object_unref(model);
  
  renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combobox), renderer, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combobox), renderer, "text", 0, NULL);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), 0);
}

// This allows the main openldev class access to the filename of the file.
string NewFileDialog::get_filename() 
{ return file_location; 
}

bool NewFileDialog::get_add_to_project()
{ return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(checkbutton));
}
