/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "project_info.h"

// The project information dialog initializes the object, set their properties,
// creates the dialog & shows it. It receives the project properties object.
ProjectInfoDialog::ProjectInfoDialog (ProjectSettings *project, 
                                      ProjectTemplate *ptemplate)
{
  // Initialize the objects
  string filenames = project->get_source_files() + project->get_header_files();
  initialize_objects();
  
  GtkCellRenderer *renderer;
  GtkListStore *model;
  GtkTreeIter iter;
  GtkTreeViewColumn *column;

  model = gtk_list_store_new(NUM_COLS, G_TYPE_STRING, G_TYPE_STRING);
  
  /* Append a row and fill in some data */
  gtk_list_store_append (model, &iter);
  gtk_list_store_set(model, &iter, COL_STATISTIC, "Project Name", COL_VALUE, 
                     project->get_name().c_str(), -1);
                     
  gtk_list_store_append (model, &iter);
  gtk_list_store_set(model, &iter, COL_STATISTIC, "Current Version", COL_VALUE, 
                     project->get_version().c_str(), -1);
                     
  gtk_list_store_append (model, &iter);
  gtk_list_store_set(model, &iter, COL_STATISTIC, "Language", COL_VALUE, 
                     ptemplate->get_language().c_str(), -1);
                     
  gtk_list_store_append (model, &iter);
  gtk_list_store_set(model, &iter, COL_STATISTIC, "Source Files", COL_VALUE, 
                     get_number_of_files(filenames).c_str(), -1);
                     
  gtk_list_store_append (model, &iter);
  gtk_list_store_set(model, &iter, COL_STATISTIC, "Lines of Code", COL_VALUE, 
                     get_lines_of_code(filenames, project->get_location()).c_str(), -1);
  
  gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(model));
  g_object_unref(model);
  
  renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Statistic", renderer,
                                           "text", COL_STATISTIC, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
  
  renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Value", renderer, 
                                           "text", COL_VALUE, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);

  gtk_window_set_default(GTK_WINDOW(dialog), close);
}

GtkWidget* ProjectInfoDialog::get_dialog()
{ return dialog;
}

// Initialize all of the widgest for the project information dialog. This
// function receives the project settings object and sets its information to
// the labels.
void ProjectInfoDialog::initialize_objects ()
{
  string gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/projectinfo.glade";
  GladeXML *xml = glade_xml_new(gladefile.c_str(), NULL, NULL);

  dialog = glade_xml_get_widget(xml, "dialog");
  treeview = glade_xml_get_widget(xml, "treeview");
  close = glade_xml_get_widget(xml, "button_close");
}

// Get the number of files in the project. This is done by counting the
// number of backslashes in the files list
string ProjectInfoDialog::get_number_of_files (string files)
{
  // Count by how many '/' characters are in the string
  int location = files.find("/", 0), count = 0;
  while (location != (signed int)string::npos)
  {
    count++;
    location = files.find("/", location+1);
  }
  
  // Convert integer properties to character arrays
  gchar buff[5];
  snprintf(buff, sizeof(buff), "%d", count);
  return string(buff);
}

// Get the number of lines of code in the project. The function receives the
// list of filenames and the project source directory location. It then opens
// each of the files & counts how many lines are in the file.
string ProjectInfoDialog::get_lines_of_code (string filenames, 
                                             string location)
{
  // Loop through to get all of the files
  int loc = filenames.find("/",0), count = 0;
  while (loc != (signed int)string::npos) 
  {
    // Create filename & then delete that file from the string
    string fn = location + string("src/") + filenames.substr(0, loc);
    filenames.erase(0, loc+1);
    loc = filenames.find("/",0);
    
    // Get the number of lines in the file
    string text = read_file(fn);
    int loc2 = text.find("\n", 0);
    while (loc2 != (signed int)string::npos)
    {
      count++;
      loc2 = text.find("\n", loc2+1);
    }
    count++;
  }
  
  // Convert integer properties to character arrays
  gchar buff[5];
  snprintf(buff, sizeof(buff), "%d", count);
  return string(buff);
}
